import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:http/http.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_contact/src/models/contact.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:contacts_service/contacts_service.dart' as contact_service;

abstract class ContactRepository {
  Future<AppResponse<bool>> syncContacts(List<Contact> contacts);
  Future<AppResponse<List<Contact>>> fetchPhoneContacts();
  Future<AppResponse<List<Contact>>> getContacts();
}

class ContactNetworkRepository extends ContactRepository {
  final BaseClient client;

  ContactNetworkRepository(
    this.client,
  );
  @override
  Future<AppResponse<List<Contact>>> fetchPhoneContacts() async {
    try {
      PermissionStatus permissionStatus = await Permission.contacts.request();
      if (!permissionStatus.isGranted) {
        return AppResponse.named(error: "Permission for contacts not granted");
      }
      List<contact_service.Contact> phoneContacts =
          (await contact_service.ContactsService.getContacts(
        withThumbnails: false,
      ))
              .toList();
      Set<Contact> contacts = Set<Contact>();
      phoneContacts.forEach((val) {
        List<String> phones = val.phones.map((number) {
          String phone = number.value
              .replaceAll(" ", "")
              .replaceAll("-", "")
              .replaceAll("(", "")
              .replaceAll(")", "");
          return phone;
        }).toList();
        List<String> emails = val.emails.map((email) {
          return email.value;
        }).toList();
        Contact contact = Contact.named(
            name: val.displayName, emails: emails, phones: phones);
        contacts.add(contact);
      });
      return AppResponse.named(data: contacts.toList());
    } catch (e) {
      debugPrint(
        "Contacts Error $e",
      );
      return AppResponse.named(error: "Can not fetch contacts");
    }
  }

  @override
  Future<AppResponse<List<Contact>>> getContacts() async {
    try {
      String url = "/contacts";
      var response = await client.get(
        url,
      );
      var json = jsonDecode(response.body);
      List<String> contacts = List<String>.from(json["contacts"] as List);
      Set<Contact> users = Set<Contact>();
      await Future.wait(contacts.map((val) async {
        AppResponse<Contact> userResponse = await _read(val);
        if (userResponse.isSuccess) users.add(userResponse.data);
      }));
      return AppResponse.named(data: users.toList());
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> syncContacts(List<Contact> contacts) async {
    try {
      List<Map<String, dynamic>> jsons = [];
      contacts.forEach((element) {
        element.phones.forEach((phone) {
          jsons.add({"name": element.name, "phone": phone});
        });
      });
      String url = "/contacts";
      await client.post(url, body: jsonEncode({"contacts": jsons}));
      return AppResponse.named(data: true);
    } catch (e) {
      debugPrint(
        "Contacts Error $e",
      );
      return AppResponse.named(error: e.toString());
    }
  }

  Future<AppResponse<Contact>> _read(String userId) async {
    try {
      String url = "/users/$userId/";
      var response = await client.get(url);
      var json = jsonDecode(response.body);
      Contact contact = Contact.named(
        id: json["id"],
        name: json["name"],
        phones: json["phone"] != null ? [json["phone"]] : [],
      );
      return AppResponse.named(data: contact);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}
